/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * This file is part of terrayazilim-json project.
 *
 * This file incorporates work covered by
 * the following copyright and permission notices:
 *
 * Copyright (C) 2018 Terra Yazılım Bilişim Hiz. Elek. Dan. Oto. ve Loj. Tic. Ltd. Şti. | info [at] terrayazilim [dot] com [dot] tr
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package tr.com.terrayazilim.json;

import java.math.BigDecimal;
import tr.com.terrayazilim.core.Objects;
import tr.com.terrayazilim.core.reflect.Primitives;

/**
 * <p>
 * {@link java.math.BigDecimal} and {@code null} values counts as {@code JsonLiteral}.
 * </p>
 * 
 * @author M.Çağrı Tepebaşılı - cagritepebasili [at] protonmail [dot] com
 * @version 1.0.3
 * @since 1.0.3
 */
public final class JsonLiteral extends JsonValue {

  private final JsonType type = JsonType.JsonLiteral;
  private final Object value;

  /**
   * @param value 
   */
  public JsonLiteral(Object value) {
    if (!isLiteral(value)) {
      throw new JsonException("Parameter is not acceptable. {Hint: ErrorCode: 111}");
    }
    
    this.value = value;
  }
  
  /**
   * @param value
   */
  public JsonLiteral(BigDecimal value) {
    this.value = value;
  }
  
  /**
   * @param value
   */
  public JsonLiteral(int value) {
    this.value = value;
  }
  
  /**
   * @param value 
   */
  public JsonLiteral(double value) {
    this.value = value;
  }
  
  /**
   * @param value 
   */
  public JsonLiteral(float value) {
    this.value = value;
  }
  
  /**
   * @param value 
   */
  public JsonLiteral(long value) {
    this.value = value;
  }
  
  /**
   * @param value 
   */
  public JsonLiteral(byte value) {
    this.value = value;
  }
  
  /**
   * @param value 
   */
  public JsonLiteral(short value) {
    this.value = value;
  }
  
  /**
   * @param value 
   */
  public JsonLiteral(Number value) {
    this.value = value;
  }
  
  /**
   * @param value 
   */
  public JsonLiteral(boolean value) {
    this.value = value;
  }
  
  /**
   * @param value 
   */
  public JsonLiteral(Boolean value) {
    this.value = value;
  }
  
  /**
   * @param value 
   */
  public JsonLiteral(String value) {
    this.value = value;
  }
  
  /**
   * @param value 
   */
  public JsonLiteral(CharSequence value) {
    this.value = value;
  }
  
  /**
   * @param value 
   */
  public JsonLiteral(char value) {
    this.value = value;
  }

  @Override
  public String toString() {
    if (Objects.nonNull(getValue())) {
      return getValue().toString();
    }
    
    return new JsonNull().toString();
  }

  /**
   * @param type
   * @return
   */
  public static boolean isLiteral(final Class<?> type) {
    Class<?> valueType = Primitives.toPrimitive(type);

    if (valueType.isAssignableFrom(String.class)) {
      return true;
    } else if (Primitives.isPrimitive(valueType)) {
      return true;
    } else if (valueType.equals(BigDecimal.class)) {
      return true;
    }
    
    return false;
  }

  /**
   * WARNING: null object counts literal since JsonNull exists as literal.
   * 
   * @param <R>
   * @param object
   * @return
   */
  public static <R> boolean isLiteral(R object) {
    if (object == null) {
      return true;
    }

    return isLiteral(object.getClass());
  }

  /**
   * @return
   */
  public boolean isNumber() {
    if (getValue() == null) {
      return false;
    }

    return Primitives.isNumber(getValue().getClass());
  }

  /**
   * @return
   */
  public boolean isString() {
    return getNullSafeValue() instanceof String;
  }

  /**
   * @return 
   */
  private Object getNullSafeValue() {
    return (getValue() == null) ? getValue() : new JsonNull();
  }
  
  /**
   * @return
   */
  public Object getValue() {
    return value;
  }

  @Override
  public JsonType getType() {
    return this.type;
  }
}
