/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * This file is part of terrayazilim-json project.
 *
 * This file incorporates work covered by
 * the following copyright and permission notices:
 *
 * Copyright (C) 2018 Terra Yazılım Bilişim Hiz. Elek. Dan. Oto. ve Loj. Tic. Ltd. Şti. | info [at] terrayazilim [dot] com [dot] tr
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package tr.com.terrayazilim.json.codec;

import java.lang.reflect.Array;
import tr.com.terrayazilim.json.Json;
import tr.com.terrayazilim.json.JsonAdapter;
import tr.com.terrayazilim.json.JsonAdapterFactory;
import tr.com.terrayazilim.json.JsonValue;
import tr.com.terrayazilim.json.stream.JsonBuffer;
import tr.com.terrayazilim.json.stream.JsonStream;
import tr.com.terrayazilim.json.type.Token;

/**
 * @author M.Çağrı Tepebaşılı - cagritepebasili [at] protonmail [dot] com
 * @version 1.0.3
 * @since 1.0.3
 * @param <T>
 */
public class JsonArrayAdapter<T> extends JsonAdapter<T>  {

  private final JsonAdapterFactory factory = new JsonAdapterFactory() {
    @Override
    public <T> JsonAdapter<T> call(final Json json, final Token<T> token) {
      return new JsonAdapter<T>() {
        @Override
        public void write(JsonBuffer buffer, T object) {
          final int length = Array.getLength(object);
          if (length == 0) {
            buffer.beginArray().endArray();
            return;
          }
          
          buffer.beginArray();
          for (int i = 0; i < length; i++) {
            Object next = Array.get(object, i);
            Token<?> nextToken = new Token<>(next.getClass());
            JsonAdapter adapter = json.getAdapter(nextToken);
            adapter.write(buffer, next);
          }
          
          buffer.endArray();
        }

        @Override
        public T read(JsonStream stream) {
          Class<?> componentType = token.getRawType().getComponentType();
          int length = stream.size();
          
          Object stack = Array.newInstance(componentType, length);
          if (length == 0) {
            return (T) stack;
          }
          
          JsonAdapter<?> adapt = json.getAdapter(new Token<>(componentType));
          int index = 0;
          
          while (stream.next()) {
            JsonValue next = stream.value();
            Object value = adapt.read(new JsonStream(next));

            Array.set(stack, index++, value);
          }
          
          return (T) stack;
        }
      };
    }
  };
  
  private final JsonAdapter<T> adapter;
  private final Token<T> token;
  private final Json json;

  /**
   * @param json
   * @param token 
   */
  public JsonArrayAdapter(Json json, Token<T> token) {
    this.json = json;
    this.token = token;
    this.adapter = factory.call(this.json, this.token);
  }

  @Override
  public void write(JsonBuffer buffer, T object) {
    adapter.write(buffer, object);
  }

  @Override
  public T read(JsonStream stream) {
    return adapter.read(stream);
  }
}
