/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * This file is part of terrayazilim-json project.
 *
 * This file incorporates work covered by
 * the following copyright and permission notices:
 *
 * Copyright (C) 2018 Terra Yazılım Bilişim Hiz. Elek. Dan. Oto. ve Loj. Tic. Ltd. Şti. | info [at] terrayazilim [dot] com [dot] tr
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package tr.com.terrayazilim.json.codec;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.lang.reflect.WildcardType;
import java.util.Collection;
import tr.com.terrayazilim.json.Json;
import tr.com.terrayazilim.json.JsonAdapter;
import tr.com.terrayazilim.json.JsonAdapterFactory;
import tr.com.terrayazilim.json.stream.JsonBuffer;
import tr.com.terrayazilim.json.stream.JsonStream;
import tr.com.terrayazilim.json.type.ConstructorToken;
import tr.com.terrayazilim.json.type.Token;

/**
 * @author M.Çağrı Tepebaşılı - cagritepebasili [at] protonmail [dot] com
 * @version 1.0.3
 * @since 1.0.3
 * @param <T>
 */
public class JsonCollectionAdapter<T> extends JsonAdapter<Collection<T>> {

  private final JsonAdapter<T> adapter;
  private final Token<T> token;
  private final Json json;

  /**
   * @param json
   * @param token
   */
  public JsonCollectionAdapter(Json json, Token<T> token) {
    this.json = json;
    this.token = token;
    this.adapter = new JsonAdapterFactory() {

      @Override
      public <R> JsonAdapter<R> call(final Json json, final Token<R> token) {
        return (JsonAdapter<R>) new JsonAdapter<Collection<R>>() {
          @Override
          public void write(JsonBuffer buffer, Collection<R> object) {
            buffer.beginArray();

            for (R next : object) {
              JsonAdapter adapt = json.getAdapter(new Token(next.getClass()));
              adapt.write(buffer, next);
            }

            buffer.endArray();
          }

          @Override
          public Collection<R> read(JsonStream stream) {
            Collection<R> stack = (Collection<R>) ConstructorToken.getCollection(token);
            int length = stream.size();
            if (length == 0) {
              return stack;
            }

            Type typeToken = getType(token.getType(), token.getRawType());
            JsonAdapter adapter = json.getAdapter(new Token<R>(typeToken));
            while (stream.next()) {
              R value = (R) adapter.read(new JsonStream(stream.value()));
              stack.add(value);
            }

            return stack;
          }
        };
      }

    }.call(this.json, this.token);
  }

  /**
   * @param context
   * @param contextRawType
   * @return
   */
  private static Type getType(Type context, Class<?> contextRawType) {
    Type collectionType = Token.getSupertype(context, contextRawType, Collection.class);

    if (collectionType instanceof WildcardType) {
      collectionType = ((WildcardType) collectionType).getUpperBounds()[0];
    }
    if (collectionType instanceof ParameterizedType) {
      return ((ParameterizedType) collectionType).getActualTypeArguments()[0];
    }

    return Object.class;
  }

  @Override
  public void write(JsonBuffer buffer, Collection<T> object) {
    adapter.write(buffer, (T) object);
  }

  @Override
  public Collection<T> read(JsonStream stream) {
    return (Collection<T>) adapter.read(stream);
  }
}
