/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * This file is part of terrayazilim-json project.
 *
 * This file incorporates work covered by
 * the following copyright and permission notices:
 *
 * Copyright (C) 2018 Terra Yazılım Bilişim Hiz. Elek. Dan. Oto. ve Loj. Tic. Ltd. Şti. | info [at] terrayazilim [dot] com [dot] tr
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package tr.com.terrayazilim.json.codec;

import java.math.BigDecimal;
import tr.com.terrayazilim.json.JsonAdapter;
import tr.com.terrayazilim.json.stream.JsonBuffer;
import tr.com.terrayazilim.json.stream.JsonStream;

/**
 * @author M.Çağrı Tepebaşılı - cagritepebasili [at] protonmail [dot] com
 * @version 1.0.3
 * @since 1.0.3
 */
public class JsonNumberAdapter extends JsonAdapter<Number> {

  @Override
  public void write(JsonBuffer buffer, Number object) {
    buffer.value(object);
  }

  @Override
  public Number read(JsonStream stream) {
    return new NumberImpl(stream.nextString());
  }
  
  /**
   * @version 1.0.3
   * @since 1.0.3
   */
  static class NumberImpl extends Number {

    private final BigDecimal value;

    public NumberImpl(String valueString) {
      this.value = new BigDecimal(valueString);
    }
    
    @Override
    public int intValue() {
      return value.intValue();
    }

    @Override
    public long longValue() {
      return value.longValue();
    }

    @Override
    public float floatValue() {
      return value.floatValue();
    }

    @Override
    public double doubleValue() {
      return value.doubleValue();
    }
  }
}
