/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * This file is part of terrayazilim-json project.
 *
 * This file incorporates work covered by
 * the following copyright and permission notices:
 *
 * Copyright (C) 2018 Terra Yazılım Bilişim Hiz. Elek. Dan. Oto. ve Loj. Tic. Ltd. Şti. | info [at] terrayazilim [dot] com [dot] tr
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package tr.com.terrayazilim.json.stream;

import java.io.StringWriter;
import tr.com.terrayazilim.core.Objects;
import tr.com.terrayazilim.json.JsonException;
import tr.com.terrayazilim.json.JsonValue;

/**
 * @author M.Çağrı Tepebaşılı - cagritepebasili [at] protonmail [dot] com
 * @version 1.0.3
 * @since 1.0.3
 */
public class JsonBuffer extends JsonWriter {

  /**
   * @version 1.0.3
   * @since 1.0.3
   */
  static public enum Status {
    INITIAL,
    ARRAY,
    KEY,
    OBJECT,
    DONE
  }

  public JsonBuffer() {
    super(new StringWriter());
  }

  /**
   * @param w 
   */
  public JsonBuffer(Appendable w) {
    super(w);
  }
  
  /**
   * @param json
   * @return 
   */
  public static JsonBuffer fromJson(JsonValue json) {
    StringWriter writer = new StringWriter();
    String text = json.toString();
    
    writer.write(text);
    
    return new JsonBuffer(writer);
  }
  
  /**
   * @return 
   */
  public Status getStatus() {
    if (this.mode == 'i') {
      return Status.INITIAL;
    } else if (this.mode == 'a') {
      return Status.ARRAY;
    } else if (this.mode == 'k') {
      return Status.KEY;
    } else if (this.mode == 'o') {
      return Status.OBJECT;
    } else if (this.mode == 'd') {
      return Status.DONE;
    }
    
    throw new JsonException("Unknown Status. {Hint: ErrorCode: 112}");
  }
  
  /**
   * @return 
   */
  public boolean isEmpty() {
    return Objects.isNull(toString());
  }
  
  @Override
  public String toString() {
    return this.mode == 'd' ? this.writer.toString() : null;
  }
}
