/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * This file is part of terrayazilim-json project.
 *
 * This file incorporates work covered by
 * the following copyright and permission notices:
 *
 * Copyright (C) 2018 Terra Yazılım Bilişim Hiz. Elek. Dan. Oto. ve Loj. Tic. Ltd. Şti. | info [at] terrayazilim [dot] com [dot] tr
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package tr.com.terrayazilim.json.type;

import java.lang.reflect.Type;
import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Deque;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import tr.com.terrayazilim.core.reflect.Constructors;

/**
 * @author M.Çağrı Tepebaşılı - cagritepebasili [at] protonmail [dot] com
 * @version 1.0.3
 * @since 1.0.3
 * @param <T>
 */
public class ConstructorToken<T> {
  
  private final Token<T> token;

  /**
   * @param type 
   */
  public ConstructorToken(Type type) {
    java.util.Objects.requireNonNull(type);
    
    this.token = new Token<>(type);
  }
  
  /**
   * @param <T>
   * @param token
   * @return 
   */
  public static <T> T getCollection(Token<T> token) {
    if (Collection.class.isAssignableFrom(token.getRawType())) {
      if (List.class.isAssignableFrom(token.getRawType())) {
        if (ArrayList.class.isAssignableFrom(token.getRawType())) {
          return (T) new ArrayList<>();
        } else if (LinkedList.class.isAssignableFrom(token.getRawType())) {
          return (T) new LinkedList<>();
        }
      } else if (Set.class.isAssignableFrom(token.getRawType())) {
        if (HashSet.class.isAssignableFrom(token.getRawType())) {
          return (T) new HashSet<>();
        } else if (TreeSet.class.isAssignableFrom(token.getRawType())) {
          return (T) new TreeSet<>();
        } else if (LinkedHashSet.class.isAssignableFrom(token.getRawType())) {
          return (T) new LinkedHashSet<>();
        }
      } else if (ArrayDeque.class.isAssignableFrom(token.getRawType())) {
        return (T) new ArrayDeque<>();
      } else if (Queue.class.isAssignableFrom(token.getRawType())) {
        return (T) new LinkedList<>();
      } else if (Deque.class.isAssignableFrom(token.getRawType())) {
        return (T) new LinkedList<>();
      } else {
        return (T) new ArrayList<>();
      }
    }
    
    return null;
  }
  
  /**
   * @param <T>
   * @param token
   * @return 
   */
  public static <T> T getMap(Token<T> token) {
    if (Map.class.isAssignableFrom(token.getRawType())) {
      if (ConcurrentMap.class.isAssignableFrom(token.getRawType())) {
        return (T) new ConcurrentHashMap<>();
      } else if (SortedMap.class.isAssignableFrom(token.getRawType())) {
        return (T) new TreeMap<>();
      } else if (LinkedHashMap.class.isAssignableFrom(token.getRawType())) {
        return (T) new LinkedHashMap<>();
      } else if (HashMap.class.isAssignableFrom(token.getRawType())) {
        return (T) new HashMap<>();
      } else {
        return (T) new LinkedHashMap<>();
      }
    } else if (Map.class.equals(token.getRawType())) {
      return (T) new LinkedHashMap<>();
    }
    
    return null;
  }

  /**
   * @return 
   */
  public T getDefaultConstructor() {
    return (T) Constructors.invoke(getToken().getRawType());
  }
  
  /**
   * @return 
   */
  public Token<T> getToken() {
    return token;
  }
}
