/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * This file is part of terrayazilim-json project.
 *
 * This file incorporates work covered by
 * the following copyright and permission notices:
 *
 * Copyright (C) 2018 Terra Yazılım Bilişim Hiz. Elek. Dan. Oto. ve Loj. Tic. Ltd. Şti. | info [at] terrayazilim [dot] com [dot] tr
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package tr.com.terrayazilim.json.type;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import tr.com.terrayazilim.core.reflect.Fields;
import tr.com.terrayazilim.json.annotation.Adapter;
import tr.com.terrayazilim.json.annotation.Deserializer;
import tr.com.terrayazilim.json.annotation.Ignored;
import tr.com.terrayazilim.json.annotation.Serializer;

/**
 * @author M.Çağrı Tepebaşılı - cagritepebasili [at] protonmail [dot] com
 * @version 1.0.3
 * @since 1.0.3
 */
public class FieldToken implements JavaToken {
  
  private final Class<?>[] presentAnnotationsTypes = new Class<?>[] {
    Serializer.class, Deserializer.class, Adapter.class, Ignored.class
  };
  
  private final Field field;

  /**
   * @param field 
   */
  public FieldToken(Field field) {
    this.field = field;
  }
  
  /**
   * @return 
   */
  public boolean isSignedWithAnnotation() {
    Annotation[] annotations = field.getDeclaredAnnotations();
    
    for (Annotation annotation : annotations) {
      for (Class<?> annotationType : presentAnnotationsTypes) {
        if (annotation.annotationType().equals(annotationType)) {
          return true;
        }
      }
    }
    
    return false;
  }
  
  /**
   * @return 
   */
  public Serializer getSignedSerializer() {
    Annotation[] annotations = field.getDeclaredAnnotations();
    
    for (Annotation annotation : annotations) {
      if (annotation.annotationType().equals(Serializer.class)) {
        Serializer serializer = (Serializer) annotation;
        return serializer;
      }
    }
    
    return null;
  }
  
  /**
   * @return 
   */
  public Deserializer getSignedDeserializer() {
    Annotation[] annotations = field.getDeclaredAnnotations();
    
    for (Annotation annotation : annotations) {
      if (annotation.annotationType().equals(Deserializer.class)) {
        Deserializer deserializer = (Deserializer) annotation;
        return deserializer;
      }
    }
    
    return null;
  }
  
  /**
   * @return 
   */
  public Adapter getSignedAdapter() {
    Annotation[] annotations = field.getDeclaredAnnotations();
    
    for (Annotation annotation : annotations) {
      if (annotation.annotationType().equals(Adapter.class)) {
        Adapter adapter = (Adapter) annotation;
        return adapter;
      }
    }
    
    return null;
  }
  
  /**
   * @return 
   */
  public Ignored getSignedIgnored() {
    Annotation[] annotations = field.getDeclaredAnnotations();
    
    for (Annotation annotation : annotations) {
      if (annotation.annotationType().equals(Ignored.class)) {
        Ignored ignored = (Ignored) annotation;
        return ignored;
      }
    }
    
    return null;
  }
  
  @Override
  public Class<?> getType() {
    return getField().getType();
  }

  @Override
  public Class<?> getRawType() {
    return Token.rawTypeOf(getType());
  }
  
  @Override
  public boolean isInterface() {
    return getType().isInterface();
  }

  @Override
  public boolean isAbstract() {
    return Modifier.isAbstract(getClass().getModifiers());
  }
  
  @Override
  public boolean isEnum() {
    return getType().isEnum();
  }

  @Override
  public boolean isArray() {
    return getType().isArray();
  }
  
  /**
   * @return 
   */
  public int getModifiers() {
    return getField().getModifiers();
  }
  
  /**
   * @param instance
   * @return
   */
  public Object getValue(Object instance) {
    return Fields.get(instance, field);
  }
  
  /**
   * @return 
   */
  public Field getField() {
    return field;
  }

  /**
   * @return 
   */
  public String getName() {
    return getField().getName();
  }
}
